@@ -12,10 +12,12 @@ __all__ = ["DetectorList", "DetectorWindow"]
 
 class DetectorList(Effect):
     """
+    A description of detector positions and properties
 
     Examples
     --------
     ::
+
         - name : full_detector
           class : DetectorList
           kwargs :
@@ -26,11 +28,26 @@ class DetectorList(Effect):
 
     def __init__(self, **kwargs):
         super(DetectorList, self).__init__(**kwargs)
-        self.meta["z_order"] = [90, 290, 390, 490]
-        self.meta["pixel_scale"] = "!INST.pixel_scale"      # arcsec
-        self.meta["active_detectors"] = "all"
+        params = {"z_order": [90, 290, 390, 490],
+                  "pixel_scale": "!INST.pixel_scale",      # arcsec
+                  "active_detectors": "all",
+                  "report_plot_include": True,
+                  "report_table_include": True}
+        self.meta.update(params)
         self.meta.update(kwargs)
 
+        # for backwards compatibility
+        new_colnames = {"xhw": "x_size", "yhw": "y_size", "pixsize": "pixel_size"}
+        mult_cols = {"xhw": 2., "yhw": 2., "pixsize": 1.}
+        if isinstance(self.table, Table):
+            for col in new_colnames:
+                if col in self.table.colnames:
+                    self.table[col] = self.table[col] * mult_cols[col]
+                    self.table.rename_column(col, new_colnames[col])
+        if "xhw_unit" in self.meta or "yhw_unit" in self.meta:
+            self.meta["x_size_unit"] = self.meta["xhw_unit"]
+            self.meta["y_size_unit"] = self.meta["yhw_unit"]
+
     def fov_grid(self, which="edges", **kwargs):
         """Returns an ApertureMask object. kwargs are "pixel_scale" [arcsec]"""
         aperture_mask = None
@@ -53,14 +70,21 @@ class DetectorList(Effect):
     @property
     def image_plane_header(self):
         tbl = self.active_table
-        pixel_size = np.min(utils.quantity_from_table("pixsize", tbl, u.mm))
+        pixel_size = np.min(utils.quantity_from_table("pixel_size", tbl, u.mm))
         x_unit = utils.unit_from_table("x_cen", tbl, u.mm)
         y_unit = utils.unit_from_table("y_cen", tbl, u.mm)
 
-        x_det_min = np.min(tbl["x_cen"] - tbl["xhw"]) * x_unit
-        x_det_max = np.max(tbl["x_cen"] + tbl["xhw"]) * x_unit
-        y_det_min = np.min(tbl["y_cen"] - tbl["yhw"]) * y_unit
-        y_det_max = np.max(tbl["y_cen"] + tbl["yhw"]) * y_unit
+        xcen, ycen = tbl["x_cen"], tbl["y_cen"]
+        dx, dy = 0.5 * tbl["x_size"], 0.5 * tbl["y_size"]
+
+        scale_factor = 1
+        if x_unit.name == "pix":
+            scale_factor = pixel_size / u.pix
+
+        x_det_min = np.min(xcen - dx) * x_unit * scale_factor
+        x_det_max = np.max(xcen + dx) * x_unit * scale_factor
+        y_det_min = np.min(ycen - dy) * y_unit * scale_factor
+        y_det_max = np.max(ycen + dy) * y_unit * scale_factor
 
         x_det = [x_det_min.to(u.mm).value, x_det_max.to(u.mm).value]
         y_det = [y_det_min.to(u.mm).value, y_det_max.to(u.mm).value]
@@ -83,20 +107,28 @@ class DetectorList(Effect):
             raise ValueError("Could not determine which detectors are active: "
                              "{}, {}, ".format(self.meta["active_detectors"],
                                                self.table))
+        tbl = utils.from_currsys(tbl)
+
         return tbl
 
     def detector_headers(self, ids=None):
         if ids is not None and all([isinstance(ii, int) for ii in ids]):
             self.meta["active_detectors"] = list(ids)
 
+        tbl = utils.from_currsys(self.active_table)
         hdrs = []
-        for row in self.active_table:
+        for row in tbl:
+            pixel_size = row["pixel_size"]
             xcen, ycen = row["x_cen"], row["y_cen"]
-            dx, dy = row["xhw"], row["yhw"]
-            cdelt = row["pixsize"]
+            dx, dy = 0.5 * row["x_size"], 0.5 * row["y_size"]
+
+            if "pix" in self.meta["x_cen_unit"]:
+                xcen, ycen = xcen * pixel_size, ycen * pixel_size
+            if "pix" in self.meta["x_size_unit"]:
+                dx, dy = dx * pixel_size, dy * pixel_size
 
             hdr = header_from_list_of_xy([xcen-dx, xcen+dx], [ycen-dy, ycen+dy],
-                                         pixel_scale=cdelt, wcs_suffix="D")
+                                         pixel_scale=pixel_size, wcs_suffix="D")
             if abs(row["angle"]) > 1E-4:
                 sang = np.sin(row["angle"] / 57.29578)
                 cang = np.cos(row["angle"] / 57.29578)
@@ -117,16 +149,20 @@ class DetectorList(Effect):
     def plot(self):
         import matplotlib.pyplot as plt
 
+        fig = plt.gcf()
+
         for hdr in self.detector_headers():
             x_mm, y_mm = calc_footprint(hdr, "D")
             x_cen, y_cen = np.average(x_mm), np.average(y_mm)
             x_mm = list(x_mm) + [x_mm[0]]
             y_mm = list(y_mm) + [y_mm[0]]
-            plt.plot(x_mm, y_mm)
-            plt.text(x_cen, y_cen, hdr["ID"])
+            plt.gca().plot(x_mm, y_mm)
+            plt.gca().text(x_cen, y_cen, hdr["ID"])
 
         plt.gca().set_aspect("equal")
 
+        return fig
+
 
 class DetectorWindow(DetectorList):
     """
@@ -135,7 +171,7 @@ class DetectorWindow(DetectorList):
     Parameters
     ----------
     pixel_size : float
-        [mm / pix] Physical pixel size
+        [mm pixel-1] Physical pixel size
     x, y : float
         [mm] Position of window centre relative to optical axis
     width, height=None : float
@@ -144,17 +180,33 @@ class DetectorWindow(DetectorList):
         [deg] Rotation of window
     gain : float, optional
         [ADU/e-]
+    units : str, optional
+        [mm, pixel] Default "mm". Sets the input parameter units.
+        If ``"pixel"``, (``x``, ``y``, ``width``, ``height``) are multiplied
+        by ``pixel_size``
 
     """
     def __init__(self, pixel_size, x, y, width, height=None, angle=0, gain=1,
-                 **kwargs):
+                 units="mm", **kwargs):
+
         if height is None:
             height = width
-        tbl = Table(data=[[0], [x], [y], [width / 2.], [height / 2.],
-                          [angle], [gain], [pixel_size]],
-                    names=["id", "x_cen", "y_cen", "xhw", "yhw",
-                           "angle", "gain", "pixsize"])
-        if "image_plane_id" not in kwargs:
-            kwargs["image_plane_id"] = 0
 
-        super(DetectorWindow, self).__init__(table=tbl, **kwargs)
+        params = {"orig_units": units,
+                  "x_cen_unit": units,
+                  "y_cen_unit": units,
+                  "x_size_unit": units,
+                  "y_size_unit": units,
+                  "pixel_size_unit": "mm",
+                  "angle_unit": "deg",
+                  "gain_unit": "electron/adu",
+                  "image_plane_id": 0}
+        params.update(kwargs)
+
+        tbl = Table(data=[[0], [x], [y], [width], [height],
+                          [angle], [gain], [pixel_size]],
+                    names=["id", "x_cen", "y_cen", "x_size", "y_size",
+                           "angle", "gain", "pixel_size"])
+        tbl.meta.update(params)
+
+        super(DetectorWindow, self).__init__(table=tbl, **params)
