@@ -72,6 +72,7 @@ class OpticalTrain:
         self.image_planes = []
         self.detector_arrays = []
         self.yaml_dicts = None
+        self._last_source = None
 
         if cmds is not None:
             self.load(cmds)
@@ -105,7 +106,7 @@ class OpticalTrain:
 
         Parameters
         ----------
-        kwargs : **dict
+        kwargs : expanded dict
             Any keyword-value pairs from a config file
 
         """
@@ -127,14 +128,13 @@ class OpticalTrain:
         orig_source : Source
         update : bool
             Reload optical system
-        kwargs : **dict
+        kwargs : expanded dict
             Any keyword-value pairs from a config file
 
         Notes
         -----
         How the list of Effects is split between the 5 main tasks:
 
-        .. todo:: List is out of date - update
         - Make a FOV list - z_order = 0..99
         - Make a image plane - z_order = 100..199
         - Apply Source altering effects - z_order = 200..299
@@ -142,6 +142,8 @@ class OpticalTrain:
         - Apply FOV-independent (2D) effects - z_order = 400..499
         - [Apply detector plane (0D, 2D) effects - z_order = 500..599]
 
+        .. todo:: List is out of date - update
+
         """
         if update:
             self.update(**kwargs)
@@ -158,6 +160,8 @@ class OpticalTrain:
         fovs = self.fov_manager.fovs
         for fov_i, fov in enumerate(fovs):
             # print("FOV", fov_i+1, "of", n_fovs, flush=True)
+            # .. todo: possible bug with bg flux not using plate_scale
+            #          see fov_utils.combine_imagehdu_fields
             fov.extract_from(source)
             fov.view()
 
@@ -172,12 +176,16 @@ class OpticalTrain:
             for ii in range(len(self.image_planes)):
                 self.image_planes[ii] = effect.apply_to(self.image_planes[ii])
 
+        self._last_source = source
+
     def readout(self, filename=None, **kwargs):
         """
+        Produces detector readouts for the observed image
 
         Parameters
         ----------
-        filename : str
+        filename : str, optional
+            Where to save the FITS file
         kwargs
 
         Returns
@@ -220,3 +228,21 @@ class OpticalTrain:
     def __setitem__(self, key, value):
         self.optics_manager[key] = value
 
+    def report(self):
+        pass
+        # user commands report
+        #   package dependencies
+        #   modes names
+        #   default modes
+        #   yaml hierarchy
+        #
+        # optics_manager
+        #   derived properties
+        #   system transmission curve
+        #   list of effects
+        #
+        # etc
+        #   limiting magnitudes
+        #
+
+
