@@ -175,6 +175,9 @@ def download_package(pkg_path, save_dir=None, url=None, from_cache=None):
         if save_dir is None:
             save_dir = rc.__config__["!SIM.file.local_packages_path"]
 
+        if not os.path.exists(save_dir):
+            os.mkdir(save_dir)
+
         try:
             if from_cache is None:
                 from_cache = rc.__config__["!SIM.file.use_cached_downloads"]
