@@ -6,13 +6,16 @@ import os
 import sys
 import warnings
 from collections import OrderedDict
+from docutils.core import publish_string
+from copy import deepcopy
 
 import yaml
 import numpy as np
 from astropy import units as u
 from astropy.io import fits
 from astropy.io import ascii as ioascii
-from astropy.table import Column
+from astropy.table import Column, Table
+from astropy.table.pprint import TableFormatter
 
 from . import rc
 
@@ -561,6 +564,18 @@ def convert_table_comments_to_dict(tbl):
     return comments_dict
 
 
+def table_to_rst(tbl, indent=0):
+    tbl_fmtr = TableFormatter()
+    lines, outs = tbl_fmtr._pformat_table(tbl, max_width=-1, max_lines=-1)
+    lines[1] = lines[1].replace("-", "=")
+    lines = [lines[1]] + lines + [lines[1]]
+
+    indent = " " * indent
+    rst_str = indent + ("\n" + indent).join(lines)
+
+    return rst_str
+
+
 def change_table_entry(tbl, col_name, new_val, old_val=None, position=None):
 
     offending_col = list(tbl[col_name].data)
@@ -667,6 +682,8 @@ def quantify(item, unit):
 
     """
 
+    if isinstance(item, str) and item[0] == "!":
+        item = from_currsys(item)
     if isinstance(item, u.Quantity):
         quant = item.to(u.Unit(unit))
     else:
@@ -854,11 +871,24 @@ def from_currsys(item):
     """
     Returns the current value of a bang-string from rc.__currsys__
     """
+    if isinstance(item, Table):
+        tbl_dict = {col: item[col].data for col in item.colnames}
+        tbl_dict = from_currsys(tbl_dict)
+        item = Table(data=[tbl_dict[key] for key in tbl_dict],
+                     names=tbl_dict.keys(),
+                     meta=item.meta)
+
+    if isinstance(item, np.ndarray) and not isinstance(item, u.Quantity):
+        item = np.array([from_currsys(x) for x in item])
+
+    if isinstance(item, list):
+        item = [from_currsys(x) for x in item]
+
     if isinstance(item, dict):
         for key in item:
             item[key] = from_currsys(item[key])
 
-    if isinstance(item, str) and item[0] == "!":
+    if isinstance(item, str) and len(item) and item[0] == "!":
         if item in rc.__currsys__:
             item = rc.__currsys__[item]
         else:
@@ -905,3 +935,21 @@ def interp2(x_new, x_orig, y_orig):
         y_new = np.interp(x_new, x_orig[::-1], y_orig[::-1])
 
     return y_new
+
+
+def write_report(text, filename=None, output=["rst"]):
+    """ Writes a report string to file in latex or rst format"""
+    if isinstance(output, str):
+        output = [output]
+
+    if filename is not None:
+        for fmt in output:
+            out_text = deepcopy(text)
+            if fmt.lower() == "latex":
+                out_text = publish_string(out_text, writer_name="latex")
+                out_text = out_text.decode("utf-8")
+
+            suffix = {"rst": ".rst", "latex": ".tex"}[fmt]
+            from pathlib import Path
+            with open(Path(filename).stem + suffix, "w") as f:
+                f.write(out_text)
